/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Classifier.h
    \brief Base class for classifiers

    Base class that all classifiers inherit. Specifies the generic
    interface any classifier will have and implements the methods that
    are independent of the classification method.  
*/



#include "Classifier.h"

Classifier::Classifier()
{
}


Classifier::~Classifier()
{
}




string
Classifier::estimate(FeatMatrix& fmatrix)
{
  return estimate(fmatrix, 0, fmatrix.rows());
}



string 
Classifier::estimate(FeatMatrix& fmatrix, unsigned int start, unsigned int end)
{
  unsigned int classNum;
  classNum = estimateClass(fmatrix, start, end);
  return classNames_[classNum];
}

unsigned int 
Classifier::estimateClass(FeatMatrix& fmatrix)
{
  return estimateClass(fmatrix, 0, fmatrix.rows());
}

unsigned int
Classifier::estimateClass(FeatMatrix& fmatrix, unsigned int start, unsigned int end)
{
  unsigned int r;
  unsigned int cl;
  
  ivec classVotes;
  classVotes.create(classSize_);
  unsigned int rows = fmatrix.rows();
  if ((start < 0) || (start > rows)) start = 0;
  if ((end < 0) || (end > rows)) end = rows;
  fvec vec(fmatrix.cols());
  
  for (r = start; r < end; r++)
    {
      vec = fmatrix.row(r);
      classVotes(classify(vec))++;
    }
  
  unsigned int max = 0;
  unsigned int classNum = 0;
  for (cl = 0; cl < classSize_; cl++)
    {
      /* cout << "Class " << cl << "( " << classNames_[cl]  
	   << ") vote = " << classVotes(cl) 
	   << " (" << (int) (((float)classVotes(cl) / (float)(end-start)* 100.0)) 			     << "%)" << endl;
      */
      if ((unsigned int)classVotes(cl) > max)
	{
	  max = classVotes(cl);
	  classNum = cl;
	}
    }
  return classNum;
}


void 
Classifier::confusion(FeatMatrix* featMatrix, fmatrix& confusionMatrix)
{
  unsigned int classSize = featMatrix->labelSize(0);
  assert(confusionMatrix->rows() == classSize);
  assert(confusionMatrix->cols() == classSize);
  assert(classSize == classSize_);
  
  unsigned int r,c;
  unsigned int rows = featMatrix->rows();
  unsigned int cols = featMatrix->cols();
  unsigned int temp;
  unsigned int wrong =0;

  fvec vec(cols);
  unsigned int cl;
  fvec classSizes;
  classSizes.create(classSize);
  confusionMatrix.setval(0.0);
  
  for (r=0; r < rows; r++)
    {
      vec = featMatrix->row(r);
      cl = featMatrix->labelNum(r,0);
      temp  = classify(vec);
      
      confusionMatrix(temp, cl)++;
      classSizes(cl)++;
    }
  
  for (r=0; r < classSize; r++)
    {
      for (c=0; c < classSize; c++)
	confusionMatrix(r,c) /= classSizes(c);
    }
 
}



float 
Classifier::evaluate(FeatMatrix* featMatrix)
{
  unsigned int classSize = featMatrix->labelSize(0);

  if (classSize != classSize_)
  {
    cerr << "FeatMatrix used for training and FeatMatrix for evaluation have different # of classes" << endl;
    cerr << "Training size = " << classSize_ << endl;
    cerr << "Eval size = " << classSize << endl;
    return 1.0;
  }

  
  unsigned int r;
  unsigned int rows = featMatrix->rows();
  unsigned int cols = featMatrix->cols();
  unsigned int temp;
  unsigned int wrong =0;
  
  fvec vec(cols);
  for (r=0; r < rows; r++)
    {
      vec = featMatrix->row(r);
      
      temp  = classify(vec);
      if (featMatrix->labelNum(r,0) != temp)
	{
	  wrong++;
	}
    }
  if (wrong == 0.0) return 0.0;
  else 
    return (wrong * 1.0/ rows * 1.0);
}

unsigned int 
Classifier::classSize()
{
  return classSize_;
}

string
Classifier::className(unsigned int i)
{
  return classNames_[i];
}

 
void
Classifier::classifyFeatMatrix(FeatMatrix& fmatrix)
{
  unsigned int classSize;
  unsigned int i;
  unsigned int size;
  
  classSize = fmatrix.labelSize(0);
  size = fmatrix.rows();
  
  fvec vec(fmatrix.cols());
  for(i=0; i < size; i++)
    {
      vec = fmatrix.row(i);
      fmatrix.setLabel(i,0,classify(vec));
    }
}


vector<string> 
Classifier::rankList(FeatMatrix& qmatrix, unsigned int listSize)
{
  unsigned int i;
  unsigned int k;
  
  vector<string> entries;
  ivec votes(classSize_);
  votes.setval(0);
  
  /* Can not have ranked list larger than number of files 
     in collection */
  if (listSize >= classSize_)
    listSize = classSize_;
  
  entries.reserve(100);
  unsigned int est;
  for (i=0; i < listSize; i++)
    {
      est = estimateClass(qmatrix);
      entries.push_back(classNames_[est]);
      mask(est);
    }
  
  return entries;
}


  
void 
Classifier::mask(unsigned int classNum)
{
  mask_(classNum) = 0;
}






