/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Daub4
    \brief Daubechies4 WaveletStep

   Applies the Daubechies 4-coefficient wavelet filter as 
a WaveletStep for the WaveletPyramid algorithm. The code is 
is based on the Numerical Recipies wavelet code. 
*/




#include "Daub4.h"

Daub4::Daub4()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = DEFAULT_WIN_SIZE;
  direction_ = WVL_FORWARD;
  c0_ = 0.4829629131445341;
  c1_ = 0.8365163037378079;
  c2_ = 0.2241438680420143;
  c3_ = -0.1294095225512604;

  workspace_.create(inSize_);
}

Daub4::Daub4(unsigned int inSize, int direction)
{
  inSize_ = inSize;
  outSize_ = inSize;
  direction_ = direction;
  c0_ = 0.4829629131445341;
  c1_ = 0.8365163037378079;
  c2_ = 0.2241438680420143;
  c3_ = -0.1294095225512604;
  workspace_.create(inSize_);
}


void 
Daub4::process(fvec& in, fvec& out) 
{
  unsigned int nh, nh1, i, j;
  unsigned int n = processSize_;
  if (processSize_ < 4) return;
  nh = n >> 1;
  nh1 = nh + 1;

  
  workspace_.setval(0.0);
  
  // Apply Filter 
  if (direction_ == WVL_FORWARD)
    {
      for (i=0, j=0; j <= n-4; j+=2, i++)
	{
	  workspace_(i) = c0_ * in(j) + c1_ * in(j+1) +
	    c2_ * in(j+2) + c3_ * in(j+3);
	  workspace_(i+nh) = c3_ * in(j) - c2_ * in(j+1) + 
	    c1_ * in(j+2) - c0_ * in(j+3);
	}
      workspace_(i) = c0_ * in(n-2) + c1_ * in(n-1) + 
	c2_ * in(0) + c3_ * in(1);
      workspace_(i+nh) = c3_ * in(n-2) - c2_ * in(n-1) + 
	c1_ * in(0) - c0_ * in(1);
    }
  // Apply transpose filter
  else if (direction_ == WVL_INVERSE)
    {
      workspace_(0) =  c2_ * in(nh-1) + c1_ * in(n-1) + 
	c0_ * in(0) + c3_ * in(nh1-1);
      workspace_(1) = c3_ * in(nh-1) - c0_ * in(n-1) + 
	c1_ * in(0) - c2_ * in(nh1-1);
      for (i=0, j=2; i < nh-1; i++)
	{
	  workspace_(j++) = c2_ * in(i) + c1_ * in(i+nh) +
	    c0_ * in(i+1) + c3_ * in(i+nh1);
	  workspace_(j++) = c3_ * in(i) - c0_ * in(i+nh) + 
	    c1_ * in(i+1) - c2_ * in(i+nh1);
	}
    }
  
  for (i=0; i < processSize_; i++)
    {
      out(i) = workspace_(i);
    }
}



	
