/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class ExtractorDictionary.h
    \brief Dictionary of supported extractors

    ExtractorDictionary returns extractors and corresponding 
feature matrices based on a string argument which is the 
extractor name. 
*/



/* ExtractorDictionary
*/


#include "ExtractorDictionary.h"


ExtractorDictionary::ExtractorDictionary()
{
  spectral_ = NULL;
  spectral1_ = NULL;
  pitches_ = NULL;
  pitch_ = NULL;
  dist_ = NULL;
  mfcc_ = NULL;
  spectralsegm_ = NULL;
  rmpitch_ = NULL;
  fftmfcc_ = NULL;
  lpc_ = NULL;
  dwtc_ = NULL;
 
  pitchExtractor_ = NULL; 
  mfccExtractor_ = NULL;
  spectralExtractor_ = NULL;
  spectral1Extractor_ = NULL;
  pitchesExtractor_ = NULL;
  distExtractor_ = NULL;
  segmExtractor_ = NULL;
  lpcExtractor_ = NULL;
  dwtExtractor_ = NULL;
  fftmfccExtractor_ = NULL;
  sfxExtractor_ = NULL;
  beatExtractor_ = NULL;
  svspectralExtractor_ = NULL;
  svmfccExtractor_ = NULL;
  svfftmfccExtractor_ = NULL;
  svrmpitchExtractor_ = NULL;
  mpitchExtractor_ = NULL;
  genreExtractor_ = NULL;
  res_ = NULL;
  prepareNames();
}


ExtractorDictionary::ExtractorDictionary(MSignal* src)
{
  spectral_ = new Spectral(src);
  spectral1_ = new Spectral1(src->winSize(), src->zeroSize());
  pitches_ = new Pitches(src->winSize(), 2);
  dist_ = new Dist(src->winSize());
  pitch_ = new Pitch(src->winSize());
  mfcc_     = new MemMFCC(src);
  unsigned int smemSize = 10;
  spectralsegm_ = new SpectralSegm(src, smemSize);
  rmpitch_ = new MultiPitchFeature(src);


  vector <System *> fArray;
  fArray.push_back(spectral_);
  fArray.push_back(mfcc_);
  fftmfcc_ = new Parallel(fArray);

  
  unsigned int order = 10;
  lpc_ = new LPC(src, order);
  dwtc_ = new DWTC(src);
  
  /* Prepare extractors */
  spectralExtractor_ = new FeatExtractor(src, spectral_);
  spectral1Extractor_ = new FeatExtractor(src, spectral1_);
  pitchesExtractor_ = new FeatExtractor(src, pitches_);
  distExtractor_ = new FeatExtractor(src, dist_);

 
  mfccExtractor_ = new FeatExtractor(src, mfcc_);
  segmExtractor_ = new FeatExtractor(src, spectralsegm_);
  lpcExtractor_ = new FeatExtractor(src, lpc_);
  dwtExtractor_ = new FeatExtractor(src, dwtc_);
  fftmfccExtractor_ = new FeatExtractor(src, fftmfcc_);
  sfxExtractor_ = new SfxExtractor(src);
  beatExtractor_ = new BeatExtractor(src);
  svspectralExtractor_ = new SvFeatExtractor(src, spectral_);
  svmfccExtractor_ = new SvFeatExtractor(src, mfcc_);
  svfftmfccExtractor_ = new SvFeatExtractor(src, fftmfcc_);
  svrmpitchExtractor_ = new SvFeatExtractor(src, rmpitch_);
  mpitchExtractor_ = new MultiPitchExtractor(src);
  genreExtractor_ = new GenreExtractor(src);
  
  /* Create extractor dictionary */ 
  extractors_["FFT"] = spectralExtractor_;
  extractors_["SFFT"] = spectral1Extractor_;
  extractors_["PITCHES"] = pitchesExtractor_;
  extractors_["DIST"] = distExtractor_;
  extractors_["PITCH"] = pitchExtractor_;
  extractors_["FFT_SEGM"] = segmExtractor_;
  extractors_["MFCC"] = mfccExtractor_;
  extractors_["FFTMFCC"] = fftmfccExtractor_;
  extractors_["SFX"] = sfxExtractor_;
  extractors_["LPC"] = lpcExtractor_;
  extractors_["DWTC"] = dwtExtractor_;
  extractors_["BEAT"] = beatExtractor_;
  extractors_["SVFFT"] = svspectralExtractor_;
  extractors_["SVMFCC"] = svmfccExtractor_;
  extractors_["SVFFTMFCC"] = svfftmfccExtractor_;
  extractors_["SVRMPITCH"]=svrmpitchExtractor_;
  extractors_["MPITCH"] = mpitchExtractor_;
  extractors_["GENRE"] = genreExtractor_;
 
  frows_["FFT"] = src->iterations();
  frows_["SFFT"] = src->iterations();
  frows_["PITCHES"] = src->iterations();
  frows_["DIST"] = src->iterations();
  frows_["FFT_SEGM"] = src->iterations();
  frows_["MFCC"] = src->iterations();
  frows_["FFTMFCC"] = src->iterations();
  frows_["SFX"] = 1;
  frows_["LPC"] = src->iterations();
  frows_["DWTC"] = src->iterations();
  frows_["BEAT"] = 1;
  frows_["SVFFT"] = 1;
  frows_["SVRMPITCH"]=1;
  frows_["SVMFCC"] = 1;
  frows_["SVFFTMFCC"] = 1;
  frows_["MPITCH"] = 1;
  frows_["GENRE"] = 1;
  frows_["PITCH"] = src->iterations(); 
  fcols_["FFT"] = spectral_->outSize();
  fcols_["SFFT"] = spectral1_->outSize();
  fcols_["PITCHES"] = pitches_->outSize();
  fcols_["PITCH"] = pitch_->outSize();
  fcols_["DIST"] = dist_->outSize();
  fcols_["FFT_SEGM"] = spectralsegm_->outSize();
  fcols_["MFCC"] = mfcc_->outSize();  
  fcols_["FFTMFCC"] = fftmfcc_->outSize();
  fcols_["SFX"] = sfxExtractor_->outSize();
  fcols_["LPC"] = lpc_->outSize();
  fcols_["DWTC"] = dwtc_->outSize();
  fcols_["BEAT"] = beatExtractor_->outSize();
  fcols_["SVFFT"] = spectral_->outSize();
  fcols_["SVRMPITCH"]= svrmpitchExtractor_->outSize();
  fcols_["SVMFCC"] = mfcc_->outSize();
  fcols_["SVFFTMFCC"] = fftmfcc_->outSize();
  fcols_["MPITCH"] = mpitchExtractor_->outSize();
  fcols_["GENRE"] = genreExtractor_->outSize();
  res_ = NULL;
  
  prepareNames();
}



void 
ExtractorDictionary::prepareNames()
{
  extractorNames_.reserve(100);
  extractorNames_.push_back("FFT");
  extractorNames_.push_back("SFFT");
  extractorNames_.push_back("PITCHES");
  extractorNames_.push_back("DIST");
  extractorNames_.push_back("PITCH");
  extractorNames_.push_back("FFT_SEGM");
  extractorNames_.push_back("MFCC");
  extractorNames_.push_back("FFTMFCC") ;
  extractorNames_.push_back("SFX");
  extractorNames_.push_back("LPC");
  extractorNames_.push_back("DWTC");
  extractorNames_.push_back("BEAT");
  extractorNames_.push_back("SVFFT");
  extractorNames_.push_back("MPITCH");
  extractorNames_.push_back("SVFFTMFCC");
  extractorNames_.push_back("SVRMPITCH");
  extractorNames_.push_back("GENRE");
}
			   

ExtractorDictionary::~ExtractorDictionary()
{
  delete spectral_;
  delete spectral1_;
  delete pitches_;
  delete pitch_;
  delete dist_;
  delete mfcc_;
  delete spectralsegm_;
  delete rmpitch_;
  delete fftmfcc_;
  delete lpc_;
  delete dwtc_;
  delete res_;
  delete pitchExtractor_;
  delete spectralExtractor_;
  delete spectral1Extractor_;
  delete pitchesExtractor_;
  delete distExtractor_;
  delete mfccExtractor_;
  delete segmExtractor_;
  delete lpcExtractor_;
  delete dwtExtractor_;
  delete fftmfccExtractor_;
  delete sfxExtractor_;
  delete beatExtractor_;
  delete svspectralExtractor_;
  delete svmfccExtractor_;
  delete svfftmfccExtractor_;
  delete svrmpitchExtractor_;
  delete mpitchExtractor_;
  delete genreExtractor_;
}


Extractor* 
ExtractorDictionary::getExtractor(string extractorstr)
{
  map<const char*, Extractor *, ltstr>::iterator cur;
  const char *ch = extractorstr.c_str();
  cur = extractors_.find(ch);
  Extractor* it;
  if (cur == extractors_.end())
    {
      cerr << "Extractor " << extractorstr << " is not supported\n" << endl;
      return NULL;
    }
  else
    {
      it = extractors_[extractorstr.c_str()];
    }
  return it;
}

unsigned int  
ExtractorDictionary::getCols(string extractorstr)
{
  map<const char*, int, ltstr>::iterator cur;
  const char *ch = extractorstr.c_str();
  cur = frows_.find(ch);
  if (cur == frows_.end())
    {
      cerr << "Extractor " << extractorstr << " is not supported\n" << endl;
      return 0;
    }
  else
    {
      unsigned int cols = fcols_[extractorstr.c_str()];
      return cols;
    }
}


unsigned int  
ExtractorDictionary::getRows(string extractorstr)
{
  map<const char*, int, ltstr>::iterator cur;
  const char *ch = extractorstr.c_str();
  cur = frows_.find(ch);
  if (cur == frows_.end())
    {
      cerr << "Extractor " << extractorstr << " is not supported\n" << endl;
      return 0;
    }
  else
    {
      unsigned int rows = frows_[extractorstr.c_str()];
      return rows;
    }
}


FeatMatrix* 
ExtractorDictionary::getFeatMatrix(string extractorstr)
{
  map<const char*, int, ltstr>::iterator cur;
  const char *ch = extractorstr.c_str();
  cur = frows_.find(ch);
  if (cur == frows_.end())
    {
      cerr << "Extractor " << extractorstr << " is not supported\n" << endl;
      return NULL;
    }
  else
    {
      unsigned int rows = frows_[extractorstr.c_str()];
      unsigned int cols = fcols_[extractorstr.c_str()];
      res_ = new FeatMatrix(rows, cols);
      return res_;
    }
}


ostream&
operator<<(ostream& o, const ExtractorDictionary& it)
{
  unsigned int i;
  for (i=0; i < it.extractorNames_.size(); i++)
    {
      o << it.extractorNames_[i] << endl;
    }
  return o;
}



