/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class ExtractorDictionary.h
    \brief Dictionary of supported extractors

    ExtractorDictionary returns extractors and corresponding 
feature matrices based on a string argument which is the 
extractor name. 
*/


#if !defined(__ExtractorDictionary_h)
#define __ExtractorDictionary_h


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <map>
#include <string.h>
#include <iostream.h>
using namespace std;


#include "FeatMatrix.h"
#include "Classifier.h"
#include "Spectral.h"
#include "Spectral1.h"
#include "MemMFCC.h"
#include "LPC.h"
#include "SfxExtractor.h"
#include "SpectralSegm.h"
#include "DWTC.h"
#include "BeatExtractor.h"
#include "SvFeatExtractor.h"
#include "MultiPitchExtractor.h"
#include "GenreExtractor.h"
#include "MultiPitchFeature.h"
#include "Parallel.h"
#include "FeatExtractor.h" 
#include "SvFeatExtractor.h"
#include "Pitches.h"
#include "MultiPitch.h"
#include "Dist.h"
#include "Pitch.h"
 
class ExtractorDictionary
{
private:
  /* comparison function for string associative array */
  struct ltstr
  {
    bool operator()(const char* s1, const char* s2) const
    {
      return strcmp(s1, s2) < 0;
    }
  };
  
  /* Systems */ 
  Spectral *spectral_;
  Spectral1 *spectral1_;
  Pitch *pitch_;
  Pitches *pitches_;
  Dist *dist_;
  MemMFCC  *mfcc_;
  SpectralSegm *spectralsegm_;
  MultiPitchFeature *rmpitch_;
  Parallel* fftmfcc_;
  LPC* lpc_;
  DWTC* dwtc_;
  
  /* Extractors */ 
  FeatExtractor* mfccExtractor_;
  FeatExtractor* spectralExtractor_;
  FeatExtractor* spectral1Extractor_;
  FeatExtractor* pitchesExtractor_;
  FeatExtractor* distExtractor_;
  FeatExtractor* pitchExtractor_;
 
  FeatExtractor* segmExtractor_;
  FeatExtractor* lpcExtractor_;
  FeatExtractor* dwtExtractor_;
  FeatExtractor* fftmfccExtractor_;
  SfxExtractor* sfxExtractor_;
  BeatExtractor* beatExtractor_;
  SvFeatExtractor* svspectralExtractor_;
  SvFeatExtractor* svmfccExtractor_;
  SvFeatExtractor* svfftmfccExtractor_;
  SvFeatExtractor* svrmpitchExtractor_;
  MultiPitchExtractor* mpitchExtractor_;
  GenreExtractor* genreExtractor_;

  /* Feature matrix result */ 
  FeatMatrix* res_;
  
  vector<string> extractorNames_;
  
  

  map<const char *, Extractor *, ltstr> extractors_;  
  map<const char *, int, ltstr> frows_;
  map<const char *, int, ltstr> fcols_;
  
  

public:
  ExtractorDictionary();
  ExtractorDictionary(MSignal* src);
  ~ExtractorDictionary();
  void prepareNames();
  unsigned int getRows(string extractorstr);
  unsigned int getCols(string extractorstr);
  FeatMatrix* getFeatMatrix(string extractorstr);
  Extractor* getExtractor(string extractorstr);
  friend ostream& operator<<(ostream&, const ExtractorDictionary&);
};




 
#endif


	

	
