/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/** 
    \class Memory
    \brief Memory for running fvec statistics

   Compute running statistics (mean,variance) of the input 
fvec using a circular-buffer memory.
*/



#include "Memory.h"

Memory::Memory()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 2 * DEFAULT_WIN_SIZE;
  past_.create(DEFAULT_MEM_SIZE, inSize_);  
  meanRow_.create(inSize_);
  stdRow_.create(inSize_);
  end_ = 0;
}

Memory::Memory(unsigned int inSize, unsigned int memSize)
{
  inSize_ = inSize;
  outSize_ = 2 * inSize;
  memSize_ = memSize;
  past_.create(memSize_, inSize_);
  meanRow_.create(inSize_);
  stdRow_.create(inSize_);
  end_ = 0;
}

Memory::~Memory()
{
}

void 
Memory::getPerc(float perc, fvec& out)
{
  unsigned int i,j;
  if (out.size() != inSize_)
    {
      cerr << "Warning: Memory::getPerc : inSize_( " << inSize_ << ") and input window size (" << out.size() << " do not agree" << endl;
      return;
    } 
  out.setval(0.0);
  for (i=0; i< memSize_; i++)
    {
      for (j=0; j < inSize_; j++)
	{
	  if (past_(i, j) < perc * meanRow_(j))
	    out(j)++;
	}
    }
  out /= memSize_;
}


void 
Memory::process(fvec& in, fvec& out) 
{
  unsigned int i;
  
  if ((in.size() != inSize_) || (out.size() != outSize_))
    {
      cerr << "Warning: Memory::process : inSize_( " << inSize_ << ") and input window size (" << in.size() << " do not agree" << endl;
      cerr << "Warning: Memory::process : outSize_( " << outSize_ << ") and output window size (" << out.size() << " do not agree" << endl;
      return;
    } 
  past_.row(end_, in);
  end_ = (end_ + 1) % memSize_;		// Circular buffer index

  
  meanRow_ = past_.meanRow();
  stdRow_  = past_.stdRow();
  //stdRow_ = past_.varRow();
  for (i=0; i < outSize_/2; i++)
    out(i) = meanRow_(i);
  
  for (i=outSize_/2; i < outSize_; i++)
    out(i) = stdRow_(i-outSize_/2);
  
}


	

	
