/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/** 
    \class MultiPitch
    \brief Multiple pitch extractor

    Multiple pitch extractor based on the paper by 
T.Tolonen and M.Karjalainen "A computationally efficient multipitch analysis 
model IEEE Trans.on Speech and Audio Processing vol 8, num 6. 2000.
*/




#include "MultiPitch.h"

MultiPitch::MultiPitch()
{
  inSize_ = DEFAULT_WIN_SIZE;
  outSize_ = 2;
  init();
}

MultiPitch::MultiPitch(unsigned int inSize)
{
  inSize_ = inSize;
  outSize_ = 2;
  init();
}

MultiPitch::~MultiPitch()
{
  delete lpf1_;
  delete lpf2_;  
  delete hpf1_;
  delete hwr_;
  delete autocor_;
  delete pe2_;
  delete pe3_;
  delete pe4_;
}


void
MultiPitch::init()
{
  featNames_.push_back("pitch");
  featNames_.push_back("mag");

  fvec num_low(3), denom_low(3);
  fvec num_high(3), denom_high(3);

  
  //coefs are for butter(2,1000) and butter(2,1000,'high') 
  num_low(0)=0.1207; num_low(1)=0.2415; num_low(2)=0.1207;
  denom_low(0)=1.0; denom_low(1)=-0.8058; denom_low(2)=0.2888;
  
  num_high(0)=0.5236; num_high(1)=-1.0473; num_high(2)=0.5236;
  denom_high(0)=1.0; denom_high(1)=-0.8058; denom_high(2)=0.2888;  
  
  lpf1_ = new Filter(num_low, denom_low, "lpf1");
  lpf2_ = new Filter(num_low, denom_low, "lpf1");
  hpf1_ = new Filter(num_high, denom_high, "lpf1");
  
  hwr_ = new HalfWaveRectifier(inSize_);
  autocor_ = new AutoCorrelationFFT(inSize_);
  magdif_ = new AMDF(inSize_);

  pe2_ = new PeakEnhancer(inSize_, 2);
  pe3_ = new PeakEnhancer(inSize_, 3);
  pe4_ = new PeakEnhancer(inSize_, 4);

  lowp_.create(inSize_);
  highp_.create(inSize_);
  sum_.create(inSize_);
  lastthree.create(3);  
}



void 
MultiPitch::process(fvec& in, fvec& out) 
{
  assert((in.size() == inSize_) && (out.size() == outSize_));  
  
  // Break signal to high and low subband
  lpf1_->process(in, lowp_);
  hpf1_->process(in, highp_);
  
  // Half-Wave rectify low subband 
  hwr_->process(lowp_, lowp_);
  
  // Half-Wave rectify and low pass filter the high subband 
  hwr_->process(highp_, highp_);
  lpf2_->process(highp_, highp_);


  /* Periodicity detection using autocorrelation computed
     using the FFT. */ 
  autocor_->process(lowp_, lowp_);
  autocor_->process(highp_, highp_);
  
  // Sum the results of periodicity detection 
  in = highp_ + lowp_;
  //autocor_->process(in, highp_);
  //magdif_->process(in, lowp_);
  //for(int i=0; i<inSize_; i++) in(i) = highp_(i)/(lowp_(i) + 1);
  
  //ofstream autocor;
  //autocor.open("autocor");
  //for(i=0; i<inSize_; i++) autocor << in(i) << endl;
  // Enhance the peaks and half wave rectify 
  
  pe2_->process(in, in);
  pe3_->process(in, in);
  pe4_->process(in, in);
  hwr_->process(in, in);
  out(0) = 17;
  out(1) = 0.0; 
  int i;

  for(i=18; i<800; i++) if(in(i) > out(1)) 
  {
    out(0) = i;
    out(1) = in(i);
  }
}


	
