/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  SegmentorSortedPeaks
   \brief  Segment algorithm based on peak detection

   Constrain segmentation by desired number of peaks and acceptable 
peak spacing. The algorithm is described in a paper presented at 
WASPAA99 by the author and Perry Cook. The paper can be found in 
http::/www.cs.princeton.edu/~gtzan/reports.html. 
   Intuitively the signal is viewed as a trajectory of points corresponding 
to the feature vectors, in a high-dimensional space. The segmentation 
boundaries are found by detecting abrupt changes in this trajectory. 
More technically, after feature vectors are extracted, a distance signal 
is calculated by taking the Mahalanobis distance between succesive feature 
vectors. To further sharpen the peaks of the resulting distance signal the 
first derivative is taken. Finally the peaks of the distance signal correspond to the segment boundaries. The largest numPeaks_ peaks are retained as segmentation boundaries by succesive maximum calculation. Peak spacing is used 
to zero an area around the detected peak to avoid peaks that correspond 
to the same segment boundary. 
*/




#include "SegmentorSortedPeaks.h"


SegmentorSortedPeaks::SegmentorSortedPeaks()
{
}


SegmentorSortedPeaks::~SegmentorSortedPeaks()
{
  
}


void
SegmentorSortedPeaks::init(unsigned int numPeaks, unsigned int peakSpacing)
{
  numPeaks_ = numPeaks;
  peakSpacing_ = peakSpacing;
  cout << "SegmentorSortedPeaks::numPeaks = " << numPeaks_ << endl;
  cout << "SegmentorSortedPeaks::peakSpacing = " << peakSpacing_ << endl;  
}


void 
SegmentorSortedPeaks::segment(FeatMatrix& featMatrix, fvec& segmentation)
{
  unsigned int r,c,p;
  int l;
  unsigned int rows;
  unsigned int cols;
  
  rows = featMatrix.rows();
  cols = featMatrix.cols();
  
  cerr << "NumPeaks " << numPeaks_ << endl;
  cerr << "Peak spacing " << peakSpacing_ << endl;
  // Clear segmentation
  segmentation.setval(0.0);
  
  fvec peaks(rows);
  fvec distances(rows);
  
  float dist;
  float prev_dist = 0.0;
  
  fvec vec(cols);
  fvec meanRow(cols);
  fvec varRow(cols);
  fvec covarRow(cols);
  
  meanRow = featMatrix.meanRow();
  varRow = featMatrix.varRow();

  for (c=0; c< cols; c++)      
    {
      if (varRow(c) <= 0.0001) 
	covarRow(c) = 1.0; // No variance
      else
	covarRow(c) = 1.0/(varRow(c));
    }  
  

  for (r = 0; r < rows; r++)
    {
      vec = featMatrix.row(r);
      dist = mahalanobis(vec, meanRow, covarRow);
      peaks(r) = fabs(dist - prev_dist);
      distances(r) = dist;
      prev_dist = dist;
    }
  
  peaks(0) = 0.0;
  float max=0.0;
  int maxPos=0;
  int peakbw = peakSpacing_;
  
  
  for (p=0;  p < numPeaks_; p++)
    {
      for (r = 0; r < rows; r++)
	{
	  if (peaks(r) > max)
	    {
	      max = peaks(r);
	      maxPos = r;
	    }
	}
      segmentation(maxPos) = 1.0;
      for (l= maxPos - peakbw; l < maxPos + peakbw; l++)
	{
	  if (( l < (int) rows)&&( l > 0))
	    {
	      peaks(l) = 0.0;
	    }
	}
      max = 0.0;
    }
  
  // Add artificially peaks at the start and end 
  
  segmentation(0) = 1.0;
  segmentation(rows-1) = 1.0;
}










