/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class  SfxExtractor
   \brief  Single Vector feature extractor for Sound Effects

   This is an example of a specific extractor for calculating 
a single feature vector. It is used for feature calculation for sound effects 
where each file is represented by a single feature vector. 
*/


#include "SfxExtractor.h"

SfxExtractor::SfxExtractor(MSignal* in):Extractor(in)
{
  unsigned int inSize = in->winSize();
  
  in_.create(inSize);
  out_.create(1);
  res_.create(4);
  
  centroidRes_.create(1);
  rmsRes_.create(1);


  hamming_ = new Hamming(inSize, 0);
  magfft_ = new MagFFT(inSize);
  centroid_ = new Centroid(magfft_->outSize());
  rms_ = new RMS(inSize);

  
  
  vector<System *> featArray;
  featArray.push_back(hamming_);
  featArray.push_back(magfft_);
  featArray.push_back(centroid_);
  series_ = new Series(featArray);

  featSize_ = 4;
  featNames_.push_back("mCentroid");
  featNames_.push_back("vCentroid");
  featNames_.push_back("vRMS");
  featNames_.push_back("cntrdRMS");
  
}

SfxExtractor::~SfxExtractor()
{
  delete hamming_;
  delete magfft_;
  delete centroid_;
  delete rms_;
  delete series_;
}

unsigned int 
SfxExtractor::outSize() 
{
  return featSize_;
}


void
SfxExtractor::extract()
{
  extract(0, src_->iterations());
}

void 
SfxExtractor::extract(unsigned int start, unsigned int end, 
		      FeatMatrix& fmatrix) 
{
  int load_matrix = 1;
  
  if ((fmatrix.rows() != 1) || (fmatrix.cols() != res_.size()))
    {
      cerr << "SfxExtractor::iterate incorrect matrix dimensions " << endl;
      cerr << "Rows = " << fmatrix.rows() << endl;
      cerr << "Cols = " << fmatrix.cols() << endl;
      cerr << "out_.size = " << out_.size() << endl;
      load_matrix = 0;
    }
  
  
     
  rmsVec_.create(end - start);
  centroidVec_.create(end - start);
  
  unsigned int t;
  unsigned int p=0;
  fmatrix.setFeatNames(featNames_);
  
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      series_->process(in_, out_);
      rms_->process(in_, rmsRes_);
      centroidVec_(t - start) = out_(0); // Fill centroid values 
      rmsVec_(t-start) = rmsRes_(0);
      if (t % ((end-start)/5) == 0)
	{
	  cerr << p * 20  << "% ";
	  p++;
	}      
    } 
  Centroid rmsCentroid(end-start);
  rmsCentroid.process(rmsVec_, rmsRes_);
  if (load_matrix) 
    {
      res_(0) = centroidVec_.mean();
      res_(1) = centroidVec_.std();
      //res_(2) = rmsVec_.std();
      //res_(3) = rmsRes_(0);
      res_(2) = 0.0;
      res_(3) = 0.0;
      fmatrix.row(0, res_);
    }
  
  cerr << "\nFeature Extractor done" << endl;
}


void
SfxExtractor::extract(unsigned int start, unsigned int end)
{
  unsigned int t;
  for (t=start; t< end; t++)
    {
      src_->get(in_, t);
      if (t % ((end-start)/10))
	{
	  cerr << (end-start)/t * 100.0 << "% processed" << endl;
	}      
    }
}

