/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/**
   \class SoundFileIO
   \brief Interface for sound file IO
   
   Abstract base class for any type of sound file IO.
Specific IO classes for various formats like AuFileIO are children 
of this class. 
*/


#include "SoundFileIO.h"

	
SoundFileIO::SoundFileIO(string filename, int mode)
{
  filename_ = filename;
  mode_ = mode;
  sdata_ = new short[winSize_ - zeroSize_];
  cdata_ = new unsigned char[winSize_ - zeroSize_];
  sdataSize_ = winSize_ - zeroSize_;
  if (mode_ == MRS_SF_READ)
    {
      sfp_ = fopen(filename_.c_str(), "rb");
      if (sfp_ == NULL) 
	{
	  perror("SoundFileIO::SoundFileIO:fopen error");
	}
    }
  else if (mode_ == MRS_SF_WRITE)
    {
      sfp_ = fopen(filename_.c_str(), "wb");
    }
  
}

SoundFileIO::~SoundFileIO()
{
  delete sdata_;
  delete cdata_;
  if (sfp_ != NULL)
    fclose(sfp_);
}



void
SoundFileIO::initWindow(unsigned int winSize, unsigned int hopSize, 
			 unsigned skipSize, unsigned int zeroSize)
{
  winSize_ = winSize;
  hopSize_ = hopSize;
  skipSize_ = skipSize;
  zeroSize_ = zeroSize;
  delete sdata_;
  delete cdata_;
  sdata_ = new short[winSize_ - zeroSize_];
  cdata_ = new unsigned char[winSize_ - zeroSize_];
  sdataSize_ = winSize_ - zeroSize_;
}





void
SoundFileIO::debug_info()
{
  cerr << "SoundFileIO " << endl;
}

unsigned long
SoundFileIO::iterations()
{
  unsigned int res;
  res = (sizeSamples_ / winSize_)-1;
  res -= skipSize_;
  res *= (winSize_ / hopSize_);
  res += 1;
  return res;
}



/*
 * C O N V E R T   F R O M   I E E E   E X T E N D E D
 */

/*
 * Copyright (C) 1988-1991 Apple Computer, Inc.
 * All rights reserved.
 *
 * Machine-independent I/O routines for IEEE floating-point numbers.
 *
 * NaN's and infinities are converted to HUGE_VAL or HUGE, which
 * happens to be infinity on IEEE machines.  Unfortunately, it is
 * impossible to preserve NaN's in a machine-independent way.
 * Infinities are, however, preserved on IEEE machines.
 *
 * These routines have been tested on the following machines:
 *    Apple Macintosh, MPW 3.1 C compiler
 *    Apple Macintosh, THINK C compiler
 *    Silicon Graphics IRIS, MIPS compiler
 *    Cray X/MP and Y/MP
 *    Digital Equipment VAX
 *
 *
 * Implemented by Malcolm Slaney and Ken Turkowski.
 *
 * Malcolm Slaney contributions during 1988-1990 include big- and little-
 * endian file I/O, conversion to and from Motorola's extended 80-bit
 * floating-point format, and conversions to and from IEEE single-
 * precision floating-point format.
 *
 * In 1991, Ken Turkowski implemented the conversions to and from
 * IEEE double-precision format, added more precision to the extended
 * conversions, and accommodated conversions involving +/- infinity,
 * NaN's, and denormalized numbers.
 */

#ifndef HUGE_VAL
# define HUGE_VAL HUGE
#endif /*HUGE_VAL*/


void
SoundFileIO::ConvertToIeeeExtended(double num,char *bytes)
{
    int    sign;
    int expon;
    double fMant, fsMant;
    unsigned long hiMant, loMant;

    if (num < 0) {
        sign = 0x8000;
        num *= -1;
    } else {
        sign = 0;
    }

    if (num == 0) {
        expon = 0; hiMant = 0; loMant = 0;
    }
    else {
        fMant = frexp(num, &expon);
        if ((expon > 16384) || !(fMant < 1)) {    /* Infinity or NaN */
            expon = sign|0x7FFF; hiMant = 0; loMant = 0; /* infinity */
        }
        else {    /* Finite */
            expon += 16382;
            if (expon < 0) {    /* denormalized */
                fMant = ldexp(fMant, expon);
                expon = 0;
            }
            expon |= sign;
            fMant = ldexp(fMant, 32);          
            fsMant = floor(fMant); 
            hiMant = FloatToUnsigned(fsMant);
            fMant = ldexp(fMant - fsMant, 32); 
            fsMant = floor(fMant); 
            loMant = FloatToUnsigned(fsMant);
        }
    }
    
    bytes[0] = expon >> 8;
    bytes[1] = expon;
    bytes[2] = hiMant >> 24;
    bytes[3] = hiMant >> 16;
    bytes[4] = hiMant >> 8;
    bytes[5] = hiMant;
    bytes[6] = loMant >> 24;
    bytes[7] = loMant >> 16;
    bytes[8] = loMant >> 8;
    bytes[9] = loMant;
}


/****************************************************************
 * Extended precision IEEE floating-point conversion routine.
 ****************************************************************/


double 
SoundFileIO::ieee_ext2double(unsigned char *bytes)
{
  double    f;
  int    expon;
  unsigned long hiMant, loMant;

  expon = ((bytes[0] & 0x7F) << 8) | (bytes[1] & 0xFF);
  hiMant    =    ((unsigned long)(bytes[2] & 0xFF) << 24)
    |    ((unsigned long)(bytes[3] & 0xFF) << 16)
    |    ((unsigned long)(bytes[4] & 0xFF) << 8)
    |    ((unsigned long)(bytes[5] & 0xFF));
  loMant    =    ((unsigned long)(bytes[6] & 0xFF) << 24)
    |    ((unsigned long)(bytes[7] & 0xFF) << 16)
    |    ((unsigned long)(bytes[8] & 0xFF) << 8)
    |    ((unsigned long)(bytes[9] & 0xFF));
	    
  if (expon == 0 && hiMant == 0 && loMant == 0) {
    f = 0;
  }
  else {
    if (expon == 0x7FFF) {    /* Infinity or NaN */
      f = HUGE_VAL;
    }
    else {
      expon -= 16383;
      f  = ldexp(UnsignedToFloat(hiMant), expon-=31);
      f += ldexp(UnsignedToFloat(loMant), expon-=32);
    }
  }

  if (bytes[0] & 0x80)
    return -f;
  else
    return f;
}



void
SoundFileIO::write_ieee_extended(FILE *fp, double num)
{
  char buf[10];
  ConvertToIeeeExtended(num, buf);
  fwrite(buf, 1, 10, fp);
}



/* read_ieee_extended:
   Read in an ieee-extended float number of 10 bits
   */

double 
SoundFileIO::read_ieee_extended()
{
  unsigned char buf[10];
  if (fread(buf, 1, 10, sfp_) != 10)
    return MRS_ERROR;
  return ieee_ext2double(buf);
}

void 
SoundFileIO::write_float(fvec& win, unsigned long index)
{
}


void 
SoundFileIO::write_linear16(fvec& win, unsigned long index)
{
  unsigned int i;
  for (i=0; i < winSize_; i++)
    {
      sdata_[i] = htons((short)(win(i) * 32768));
    }
  if (fwrite(sdata_, sizeof(short), winSize_, sfp_) != winSize_)
    {
      cerr << "Problem: could not write window " << index << "to file " << filename_ << endl;
    }
}


void 
SoundFileIO::read_linear8(fvec& win, unsigned long index) 
{
  fseek(sfp_, index * hopSize_ + skipSize_ + sfp_begin_, SEEK_SET);
  if (fread(cdata_, sizeof(unsigned char), winSize_ - zeroSize_, sfp_) != winSize_ - zeroSize_)
    {
      cerr << "Problem: could not read window " << index << "from file " << filename_ << endl;
    }

  if (zeroSize_ == 0)
    {
      for (unsigned long i=0; i < winSize_; i++)
	{
	  win(i) = -1.0 + (float) cdata_[i] / 128;
	}
    }
}


void 
SoundFileIO::read_linear16(fvec& win, unsigned long index)
{
  unsigned int i;
  fseek(sfp_, index * hopSize_ * 2 + skipSize_ + sfp_begin_,SEEK_SET);
  if (fread(sdata_, sizeof(short), winSize_ - zeroSize_, sfp_) != winSize_ - zeroSize_)
    {
      cerr << "Problem: could not read window " << index << "from file " << filename_ << endl;
    }

  unsigned int hzeroSize_ = zeroSize_ /2;
  if (zeroSize_ == 0)
    {
      for (i=0; i < winSize_; i++)
	{
	  win(i) = (float) sdata_[i] / (float) 32768.0;
	}
    }
  else
    {
      for (i=0; i < hzeroSize_; i++)
	{
	  win(i) = 0.0;
	}
      for (i= hzeroSize_; i < winSize_ - hzeroSize_; i++)
	{
	  win(i) = (float) sdata_[i - hzeroSize_] / (float) 32768.0;
	}
      for (i= winSize_ - hzeroSize_; i < winSize_; i++)
	{
	  win(i) = 0.0;
	}
    }
}


void 
SoundFileIO::read_linear16Swap(fvec& win, unsigned long index)
{
  unsigned int i;
  fseek(sfp_, index * hopSize_ * 2 + skipSize_ + sfp_begin_,SEEK_SET);
  if (fread(sdata_, sizeof(short), winSize_ - zeroSize_, sfp_) != winSize_ - zeroSize_)
    {
      cerr << "Problem: could not read window " << index << "from file " << filename_ << endl;
    }
  unsigned int hzeroSize_ = zeroSize_ /2;
  if (zeroSize_ == 0)
    {
      for (unsigned long i=0; i < winSize_; i++)
	{
	  sdata_[i] = ntohs(sdata_[i]);
	  win(i) = (float) sdata_[i] / (float) 32768.0;
	}
    }
  else
    {
      
      for (i=0; i < hzeroSize_; i++)
	{
	  win(i) = 0.0;
	}
      for (i= hzeroSize_; i < winSize_ - hzeroSize_; i++)
	{
	  win(i) = (float) sdata_[i - hzeroSize_] / (float) 32768.0;
	}
      for (i= winSize_ - hzeroSize_; i < winSize_; i++)
	{
	  win(i) = 0.0;
	}
    }
}


void 
SoundFileIO::read_float(fvec& win, unsigned long index)
{
}

