/*
** Copyright (C) 2000 George Tzanetakis <gtzan@cs.princeton.edu>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


/**
   \class Spectral
   \brief FFT based Spectral features 

   Calculate FFT mangitude based features. 
The Centroid, Rolloff and Flux are calculated 
on the magnitude spectrum of the input fvec 
and Zcrs are calculated in time domain. 
This System feature are typically used in classification. 
*/


#include "Spectral.h"

Spectral::Spectral()
{
  inSize_ = DEFAULT_WIN_SIZE;
  zeroSize_ = 0;
  outSize_ = 9;
  init();
}

Spectral::Spectral(MSignal *src)
{
  inSize_ = src->winSize();
  zeroSize_ = src->zeroSize();
  outSize_ = 9;
  init();
}


void 
Spectral::init()
{
  temp0_.create(3);  
  temp1_.create(1);
  temp2_.create(1);
  temp3_.create(4);
  temp4_.create(8);
  temp5_.create(2);
  temp6_.create(1);
  
  int memSize = 40;
  
  
  hamming_ = new Hamming(inSize_, zeroSize_);
  magfft_  = new MagFFT(inSize_);
  db_      = new DB(magfft_->outSize(), 0.0001);
  shapeDesc_  = new ShapeDesc(magfft_->outSize());
  zcrs_ = new Zcrs(inSize_);
  rms_ = new RMS(inSize_);
  memory_ = new Memory(outSize_/2, memSize);
  lowMemory_ = new Memory(1, memSize);
  
  vector<System *> featArray;
  featArray.push_back(hamming_);
  featArray.push_back(magfft_);
  featArray.push_back(shapeDesc_);
  
  series_ = new Series(featArray);
  
  featSize_ = 9;
  featNames_.push_back("MeanCentroid");
  featNames_.push_back("MeanRolloff");
  featNames_.push_back("MeanFlux");
  featNames_.push_back("MeanZerocrossings");
  featNames_.push_back("StdCentroid");
  featNames_.push_back("StdRolloff");
  featNames_.push_back("StdFlux");
  featNames_.push_back("StdZerocrossings");
  featNames_.push_back("LowEnergy");
}



Spectral::~Spectral()
{
  delete hamming_;
  delete magfft_;
  delete db_;
  delete shapeDesc_;
  delete zcrs_;
  delete rms_;
  delete memory_;
  delete lowMemory_;
  delete series_;
}


void 
Spectral::process(fvec& in, fvec& out) 
{
  unsigned int i;
  assert((in.size() == inSize_) && (out.size() == outSize_));      
  
  series_->process(in, temp0_);
  zcrs_->process(in, temp1_);
  rms_->process(in, temp2_);
  
  for (i=0; i<3; i++)
    {
      temp3_(i) = temp0_(i);
    }
  temp3_(3) = temp1_(0);
  memory_->process(temp3_, temp4_);
  lowMemory_->process(temp2_, temp5_);
  lowMemory_->getPerc(0.50, temp6_);
  for (i=0; i<8; i++)
    out(i) = temp4_(i);
  out(8) = temp6_(0);

}


	

	
