#include "SpectralPeak.h"


SpectralPeak::SpectralPeak(int inSize)
{
  inSize_ = inSize;
	outSize_ = inSize_ / 2;
	threshhold_ = -80;
  init();
}

SpectralPeak::SpectralPeak(int inSize, int outSize)
{
  inSize_ = inSize;
	outSize_ = outSize;
	threshhold_ = -80;
  init();
}

SpectralPeak::SpectralPeak(int inSize, int outSize, float threshhold)
{
  inSize_ = inSize;
	outSize_ = outSize;
	threshhold_ = threshhold;
  init();
}

SpectralPeak::~SpectralPeak()
{
}


void
SpectralPeak::init()
{
  nBins_ = inSize_ / 2;
	
  magfft_ = new MagFFT(inSize_);
  hamming_ = new Hamming(inSize_, 0);
	db_ = new DB(nBins_, pow(10, threshhold_ / 20) / 2);
	
  windowed_.create(inSize_);
  spectrum_.create(nBins_);
	dbspectrum_.create(nBins_);
	amps_.create(nBins_);
	phases_.create(nBins_);
	partials_.create(nBins_);
	width_.create(nBins_);
	fft_.create(inSize_);
}


void
SpectralPeak::process(fvec& in, fvec& out) 
{
  int i, lastPeak, peakPosition;
	float freq, amp, phase, diff, leftPhase, rightPhase, interpolated, binWidth = 0;

  hamming_->process(in, windowed_);
  magfft_->process(windowed_, spectrum_);
	//ofstream sp("spectrum");
	//for(i = 0; i < inSize_/2; i++) sp << spectrum_(i) << endl;
	db_->process(spectrum_, dbspectrum_);
	fft_ = magfft_->fft();
	
	for (i = 1, nPartials_ = 0; i < nBins_ - 2; i++)
	{
		float leftMag = dbspectrum_(i), middleMag	= dbspectrum_(i+1), rightMag = dbspectrum_(i+2);
		if(middleMag == leftMag && leftMag == rightMag)	// local constant detected 
		{
			if(nPartials_ > 0 && width_(nPartials_-1) == 0) width_(nPartials_-1) = binWidth;
			binWidth = 1;
		}
		if(middleMag <= leftMag && middleMag <= rightMag) // local Minimum detected 
		{
			if(nPartials_ > 0 && width_(nPartials_-1) == 0) width_(nPartials_-1) = binWidth;
			binWidth = 0;
		}
		if(middleMag >= leftMag && middleMag >= rightMag && middleMag > threshhold_) // local maximum Detected 
		{
			peakPosition = i+1; 	// middleMag has index i+1
			if(nPartials_ > 0 && width_(nPartials_-1) == 0)
			{
				width_(nPartials_-1) = binWidth - (peakPosition - partials_(nPartials_-1)) / 2.0;
				binWidth = (peakPosition - partials_(nPartials_-1)) / 2.0;
			}	
			if(middleMag == leftMag && middleMag > rightMag && nPartials_ > 0)	// end of a constant area 
			{
				interpolated = partials_(nPartials_-1) + (i+1-partials_(nPartials_-1)) / 2; // center BinPos 
				amps_(nPartials_-1) = dbspectrum_(round(interpolated));
				phases_(nPartials_-1) = fft_(2 * round(interpolated) + 1);
				width_(nPartials_-1) = peakPosition - partials_(nPartials_-1);
				partials_(nPartials_-1) = interpolated; // interpolated BinPos is stored	
			}
			else
			{
				diff =  0.5 * (leftMag-rightMag) / (leftMag- 2*middleMag + rightMag);
				partials_(nPartials_) = peakPosition + diff;
				amps_(nPartials_) = middleMag - .25 * (leftMag - rightMag) * diff;
				if(diff >= 0)
				{
					leftPhase  = fft_(i*2 + 1);
					rightPhase = fft_(i*2 + 3);
				}
				else
				{
					leftPhase  = fft_(i*2 - 1);
					rightPhase = fft_(i*2 + 1);
				}
				if(fabsf(rightPhase-leftPhase) > PI)
				{
					if(rightPhase > 0) leftPhase += TWO_PI;
					else rightPhase += TWO_PI;
				}
				if(diff >= 0) phases_(nPartials_) = leftPhase + diff * (rightPhase - leftPhase);
				else phases_(nPartials_) = leftPhase + (1 + diff) * (rightPhase - leftPhase);	
				nPartials_++;
			}
		}
		binWidth++;
	}
		 
	if(nPartials_ > 0 && width_(nPartials_-1) == 0)
	{ 			
		lastPeak = partials_(nPartials_-1);
		width_(nPartials_-1) = binWidth - (i-lastPeak) / 2.0;
		binWidth = (i-lastPeak) / 2.0;
	}
	out = partials_;
}


int 
SpectralPeak::numberOfPartials()
{
  return nPartials_;
}

fvec& SpectralPeak::getSpectrum()
{
  return spectrum_;
}


fvec& SpectralPeak::getAmps()
{
  return amps_;
}


fvec& SpectralPeak::getPhases()
{
  return phases_;
}


inline int
SpectralPeak::round(float r)
{
  if(r - floor(r) <= 0.5 ) return (int)floor(r);
  return (int)ceil(r);
}











